;******************************************************************************************************
; Le 05/11/2020 "Animations-Alatoire_fluo_arc_feux" sur PIC 16F690
;
; Montage : Animations Alatoire, clairage tube fluorescent en panne, arc lectrique, voitures phares, feux pntrants et gyrophares. 
;
; Site web : http://www.la-tour.info/uts/uts_index.html
; Montage, schma, notice, typon, explication, circuit imprim, programme assembleur microchip pour pic  cette adresse internet.
; Montage  PIC 16F690 pour alimenter des charges relies au + 12 volts.
; Sert sur un rseau de train. Rubrique : Modlisme ferroviaire et lectronique locomotive maquette animation et clairage.
; Deux groupes de 8 sorties branchs sur 2 ULN2803.
; L'entre R3 est relie aux sorties via des diodes et des cavaliers pour choisir la configuration  la mise sous tension :
;
;------ RA5 RA4 RC5 RC4 : Mode (I => Cavalier en place) -------------------------------------
;        4   3   2   1
;     :              0  : 0 = Alatoires n 4 et 12
; RA3 :              I  : 1 = Flammes
; RA3 :          I      : 1 = Feux pntration voiture n 1 et 2
; RA3 :          I   I  : 1&1 = Feux pntration voiture n 1 et 2, gyrophare voiture n 3 + Phares voitures n 1, 2 et 3
; RA3 :      0          : 0 = Alatoires n 9 et 10
; RA3 :      I          : 1 = Arc n 3 + Fluo n 3
; RA3 :  0              : 0 = Alatoires n 4 et 12
; RA3 :  I              : 1 = Gyrophare + Phares voiture n 4
;
;********************************************************************************************
;
; L'utilisation des pattes du PIC 16F690
; RA0 - Entre Analogique CAN0 (0-5 Volts) = Vitesse d'animation.
; RA3 - Entre RA3 Multiplexe avec les sorties RA5, RA4, RC5 et RC4 via une diode + rsistance de rappel  la masse.
; Sorties Groupe 1
;   RA5 - Alatoire 1, Phares 1
;   RA4 - Alatoire 2, Phares 1
;   RC5 - Alatoire 3, Phares 2
;   RC4 - Alatoire 4, Phares 4
;   RC3 - Alatoire 5, Flammes 1, Feux pntration 1a
;   RC6 - Alatoire 6, Flammes 2, Feux pntration 1b
;   RC7 - Alatoire 7, Flammes 3, Feux pntration 2a
;   RB7 - Alatoire 8, Flammes 4, Feux pntration 2b
; Sorties Groupe 2
;   RA1 - Arc 1
;   RA2 - Arc 2 
;   RC0 - Arc 3, Alatoire 9
;   RC1 - Fluo 1
;   RC2 - Fluo 2
;   RB4 - Fluo 3, Alatoire 10
;   RB5 - Clignotant 3, Alatoire 11
;   RB6 - Clignotant 4, Alatoire 12
;
; Flammes = Effet de scintillement d'une flamme, divers effets succesifs.
; Utilise la routine : Calcul de valeur alatoire : http://www.piclist.com/techref/microchip/rand8bit.htm
; Utilise la routine : Calcul de valeur alatoire : https://www.dontronics.com/psbpix/random.html
; Mmoire du PIC 16F690 = 4Ko = 0x0000  0x0FFF.
; Page 0 = 0x000  0x7FF, page 0x800  0xFFF.
; Sous programme d'initialisation en 0x900 en page 1, pour laisser de la place en page 0.
; *******************************************************************
;
	LIST p=16F690, R=DEC ; Utilisation du mode dcimal.
	include	"P16F690.inc"
;	Code non protg,  oscill interne sans sortie, pas de dtection tension basse, pas d'utilisation de la patte MCLR, dmarrage lent, Watchdog hors service
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOR_OFF & _IESO_OFF & _FCMEN_OFF)
;
; _INTRC_OSC_NOCLKOUT  EQU  H'3FFC'    ; INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN
;                                      ; FOSC<2:0> = 100
; _WDT_OFF             EQU  H'3FF7'    ; WDT disabled and can be enabled by SWDTEN bit of the WDTCON register
; _PWRTE_OFF           EQU  H'3FFF'    ; PWRT disabled
; _MCLRE_OFF           EQU  H'3FDF'    ; MCLR pin function is digital input, MCLR internally tied to VDD
; _CPD_OFF             EQU  H'3FFF'    ; Data memory code protection is disabled
; _CP_OFF              EQU  H'3FFF'    ; Program memory code protection is disabled
; _BOREN_OFF           EQU  H'3CFF'    ; BOR disabled
; _IESO_OFF            EQU  H'3BFF'    ; Internal External Switchover mode is disabled
; _FCMEN_OFF           EQU  H'37FF'    ; Fail-Safe Clock Monitor is disabled
;
BANK0	macro			; MACRO : Passer en banque0.
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	endm
BANK1	macro			; Passer en banque1.
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	endm
BANK2	macro			; Passer en banque2.
	bcf	STATUS,RP0
	bsf	STATUS,RP1
	endm
;
;*************************************************************************************************************
; Dfinitions des constantes internes modifiables par l'utilisateur.
; Dlai alatoire entre les phares et les feux pntrants = 1  7 secondes.
; Dlais alatoires supplmentaires de 0  16-32 secondes sur les priodes de marche et d'arrt.
; Flashs de pntration altrns voiture n 1.
PEN_NF1	EQU	3	; Nb de flashs de suite (1  127).
PEN_DF1	EQU	20	; Dure des feux pntrants en secondes (5  110) + 0  16 secondes alatoire.
PEN_PO1	EQU	40	; Dure de la pause longue dure en secondes (5  220) + 0  32 secondes alatoire.

; Flashs de pntration altrns voiture n 2.
PEN_NF2	EQU	4	; Nb de flashs de suite (1  127).
PEN_DF2	EQU	30	; Dure des feux pntrants en secondes (5  110) + 0  16 secondes alatoire.
PEN_PO2	EQU	60	; Dure de la pause longue dure en secondes (5  220) + 0  32 secondes alatoire.

; Gyrophare clignotant voiture n 3.
CLI_DC3	EQU	40	; Dure du gyrophare clignotant en secondes (5  70) + 0  10 secondes alatoire.
CLI_PO3	EQU	80	; Dure de la pause longue dure en secondes (5  220) + 0  32 secondes alatoire.

; Gyrophare clignotant voiture n 4.
CLI_DC4	EQU	50	; Dure du gyrophare clignotant en secondes (5  70) + 0  10 secondes alatoire.
CLI_PO4	EQU	100	; Dure de la pause longue dure en secondes (5  220) + 0  32 secondes alatoire.
;
; Soudure  l'arc 1 = 10  41 squences lumineuses de suite.
ARC_DF1 EQU	35	; Dure de fonctionnement de l'arc 1,     par pas de 1 seconde de 1  255, par dfaut = 35 secs.
ARC_PO1	EQU	25	; Dure de pause longue dure de l'arc 1, par pas de 1 seconde de 1  255, par dfaut = 25 secs.

; Soudure  l'arc 2 = 5  20 squences lumineuses de suite.
ARC_DF2 EQU	30	; Dure de fonctionnement de l'arc 2,     par pas de 1 seconde de 1  255, par dfaut = 30 secs.
ARC_PO2	EQU	40	; Dure de pause longue dure de l'arc 2, par pas de 1 seconde de 1  255, par dfaut = 40 secs.

; Soudure  l'arc 3 = 3  10 squences lumineuses de suite.
ARC_DF3 EQU	20	; Dure de fonctionnement de l'arc 3,     par pas de 1 seconde de 1  255, par dfaut = 20 secs.
ARC_PO3	EQU	20	; Dure de pause longue dure de l'arc 3, par pas de 1 seconde de 1  255, par dfaut = 20 secs.

; Tube Fluo 1 = Cinq tentatives d'allumage puis reste allum un temps relativement long.
VIT_FL1	EQU	5	; Vitesse de base pour le tube fluo 1 (1 = rapide, 6 = moyen, 255 = lent).
POS_FL1	EQU	40	; Dure de pause longue dure du tube fluo 1, par pas de 1 seconde de 1  255, par dfaut = 40 secs.

; Tube Fluo 2 = Nombreuses tentatives d'allumage, mais ne reste allum qu'un court moment.
VIT_FL2	EQU	6	; Vitesse de base pour le tube fluo 2 (1 = rapide, 6 = moyen, 255 = lent).
POS_FL2	EQU	30	; Dure de pause longue dure du tube fluo 2, par pas de 1 seconde de 1  255, par dfaut = 30 secs.

; Tube Fluo 3 = Tentatives d'allumage en continu, mais ne parvient pas  s'allumer.
VIT_FL3	EQU	7	; Vitesse de base pour le tube fluo 3 (1 = rapide, 6 = moyen, 255 = lent).
POS_FL3	EQU	20	; Dure de pause longue dure du tube fluo 3, par pas de 1 seconde de 1  255, par dfaut = 20 secs.

;*************************************************************************************************************
	CBLOCK 0x20     ; La table des variables commence  0x20.
				; Mode de fonctionnement du programme = Lecture des cavaliers 1  4.
	mode_fonc1	; bit(0) =1/0 => Alatoire 1  3 et 11 = ON/OFF
				; bit(1) =1/0 => Alatoire   4 et 12 = ON/OFF
				; bit(2) =1/0 => Alatoire   5    8 = ON/OFF
				; bit(3) =1/0 => Alatoire   9 et 10 = ON/OFF

	mode_fonc2	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
				; bit(1) =1/0 => Flammes = ON/OFF
				; bit(2) =1/0 => Feux pntration 1 & 2 = ON/OFF
				; bit(3) =1/0 => Gyrophare 3 & phares 3 = ON/OFF
				; bit(4) =1/0 => Gyrophare 4 & phares 4 = ON/OFF
				; bit(5) =1/0 => Arc 3  = ON/OFF
				; bit(6) =1/0 => Fluo 3 = ON/OFF
				; bit(7) =1/0 => Phares 1 & 2 = ON/OFF

	varale1s : 3 ; Variable alatoire sur 16 bits + une variable de combinaison en +2.
	varale2s : 3 ; Variable alatoire sur 16 bits + une variable de combinaison en +2.
	cpt_can01	; Compteur pour la demande de conversion CAN.
	can_lue		; Valeur du convertisseur A/N brute.
	status_pha	; Bit(0) =0/1 => Phares 1 en pause longue / En marche
				; Bit(1) =0/1 => Feux pntrant 1 en pause longue / En marche
				; Bit(2) =0/1 => Phares 2 en pause longue / En marche
				; Bit(3) =0/1 => Feux pntrant 2 en pause longue / En marche
				; Bit(4) =0/1 => Phares 3 en pause longue / En marche
				; Bit(5) =0/1 => Gyrophare 3 en pause longue / En marche
				; Bit(6) =0/1 => Phares 4 en pause longue / En marche
				; Bit(7) =0/1 => Gyrophare 4 en pause longue / En marche
				
	cpt_feup11	; Compteur de boucle principale feux de pntration 1
	cpt_feup12	; Compteur de boucle secondaire
	cpt_feup13	; Compteur de squence lumineuse
	cpt_feup14	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash
	cpt_feup15	; Compteur de boucle en pause
	cpt_feup16	; Numro de squence de nombre de flashs

	cpt_feup21	; Compteur de boucle principale feux de pntration 2
	cpt_feup22	; Compteur de boucle secondaire
	cpt_feup23	; Compteur de squence lumineuse
	cpt_feup24	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash
	cpt_feup25	; Compteur de boucle en pause
	cpt_feup26	; Numro de squence de nombre de flashs

	cpt_clig31	; 
	cpt_clig32	; 
	cpt_clig33	; 
	cpt_clig41	; 
	cpt_clig42	; 
	cpt_clig43	; 

	cpt_arc11	; Compteurs de temporisation pour soudure  l'arc 1.
	cpt_arc12	; //
	cpt_arc13	; //
	cpt_arc14	; //
	cpt_arc15	; //
	cpt_arc16	; //
	cpt_arc17	; //
	cpt_arc18	; Etat gnral de fonctionnement de l'arc.
	cpt_arc21	; Compteurs de temporisation pour soudure  l'arc 2.
	cpt_arc22	; //
	cpt_arc23	; //
	cpt_arc24	; //
	cpt_arc25	; //
	cpt_arc26	; //
	cpt_arc27	; //
	cpt_arc28	; Etat gnral de fonctionnement de l'arc.
	cpt_arc31	; Compteurs de temporisation pour soudure  l'arc 3.
	cpt_arc32	; //
	cpt_arc33	; //
	cpt_arc34	; //
	cpt_arc35	; //
	cpt_arc36	; //
	cpt_arc37	; //
	cpt_arc38	; Etat gnral de fonctionnement de l'arc.

	status_fluo	; Bit(x)=0 => En fonctionnement longue dure.
				; Bit(x)=1 => En pause longue dure.
				; Fluo3  = Bit(3), Fluo2  = Bit(2), Fluo1 = Bit(1).
				; bit(5) =1 => Au moins une lampe alatoire en fonction. 
				; bit(6) =1 => On vient de sortir du programme d'interruption, on dmarre le programme.
				; bit(7) =1 => On vient de finir la squence PMW des flammes, on peut recalculer leurs valeurs.
	cpt_fluo11	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo12	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo13	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo14	; Compteur de squence lumineuse pour tube fluorescent. 
	cpt_fluo21	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo22	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo23	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo24	; Compteur de squence lumineuse pour tube fluorescent. 
	cpt_fluo31	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo32	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo33	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo34	; Compteur de squence lumineuse pour tube fluorescent. 

	cpt_inter	; Compteur d'interruption pour les sorties PMW.
	cpt_flam00	; Numro d'effet en cours
	cpt_flam01	; Compteur de boucle principale pour l'effet en cours
	cpt_flam02	; 1er  compteur de dlai en cours
	cpt_flam03	; 2eme compteur de dlai en cours
	cpt_flam11	; Puissance flamme n 1
	cpt_flam21	; Puissance flamme n 2
	cpt_flam31	; Puissance flamme n 3
	cpt_flam41	; Puissance flamme n 4
	cpt_flam11b	; Recopie puissance flamme n 1
	cpt_flam21b	; Recopie puissance flamme n 2
	cpt_flam31b	; Recopie puissance flamme n 3
	cpt_flam41b	; Recopie puissance flamme n 4

	cpt_vara01	; Compteur de boucle principale pour calcul des variables alatoire.

	cpt_alea01	;
	cpt_alea02	;
	cpt_alea03	;
	
	cpt_lamp01	;
	cpt_lamp05	;
	cpt_lamp06	;
	cpt_lamp07	;
	cpt_lamp08	;
	cpt_lamp11	
	cpt_lamp15	;
	
	temp1		; Variable locale
	w_temp		; Sauvegardes pour les interruptions.
	status_temp	; //
	verif_tmr0	;
	ENDC		; "7F" maxi.

	cpt_lamp14	EQU cpt_fluo32	; Dfinition d'adresses ayant deux emplois non simultans.
	cpt_lamp02	EQU cpt_clig32
	cpt_lamp03	EQU cpt_clig33
	cpt_lamp04	EQU cpt_clig42
	cpt_lamp16	EQU cpt_clig43
		
;*******************************************************************
; Dbut du programme.
;
	org		0x2100 	  			 ; Initialize EEPROM Data
	dt 		"ANIMATION ALEATOIRE FLUO ARC FEUX FLAMMES V1_____"	; Texte visible dans le '.hex'.
	dt		"Le 05/11/2020 "
	dt		"http://www.la-tour.info/uts/uts_index.html"
	ORG     0x000
	goto    init
	org     0x004
	goto    interrupt
	dt		"__ANIMATION ALEATOIRE FLUO ARC FEUX FLAMMES V1______"	; Texte visible dans le '.hex'.
	dt		"__Le 05/11/2020___"
	dt		"__http://www.la-tour.info/uts/uts_index.html__"
;
;***************************************************************************************
; Traitement des interruptions du timer TMR0 pour gnrer les sorties PWM des flammes
; A 8 Mhz Interruption TMR0 = 112 S =  224 instructions dans la boucle principale.
; Comme il y du code dans ce sous programme d'interruption, on passe dans cette routine
; toutes les 117 s + 15 s = 132 s. Ce qui donne pour une rsolution de la PMW sur 128 niveaux = 59 hz.
; On charge les valeurs de PMW ici de faon synchrone, pour viter des sursauts de lumire.
; 
interrupt
	bcf     INTCON,GIE		; Inhibe toutes les interruptions.
	movwf   w_temp  		; Sauve le registre W.
	swapf	STATUS,w		; Swap status avec rsultat dans w.
	movwf	status_temp		; Sauver le STATUS. Pas besoin de passer en BANK0, car on n'utilise ni BANK1 ni BANK2 dans la boucle principale.

	bsf		status_fluo,6	; Signal de recalcul global => 1.
	incf	cpt_inter,f		; Compteur pour les flammes 1 + 2 + 3.
	incf	cpt_inter,f
	btfsc	STATUS,Z
	bsf		status_fluo,7	; Signal de recalcul des flammes => 1.

	btfss 	mode_fonc2,1	; Flammes en marche ?
	goto	pas_flammes
	movf	cpt_inter,w		; Flamme 1
	subwf	cpt_flam11,w	; cpt_flam12 = Valeur de la puissance de la flamme 1 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf 	PORTC,3			; Allume la flamme 1 PWM.
	btfss	STATUS,C
	bcf 	PORTC,3			; Eteint la flamme 1 PWM.
	movf	cpt_inter,w		; Flamme 2
	subwf	cpt_flam21,w	; cpt_flam22 = Valeur de la puissance de la flamme 2 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf 	PORTC,6			; Allume la flamme 2 PWM.
	btfss	STATUS,C
	bcf 	PORTC,6			; Eteint la flamme 2 PWM.
	movf	cpt_inter,w		; Flamme 3
	subwf	cpt_flam31,w	; cpt_flam32 = Valeur de la puissance de la flamme 3 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf 	PORTC,7			; Allume la flamme 3 PWM.
	btfss	STATUS,C
	bcf 	PORTC,7			; Eteint la flamme 3 PWM.
	movf	cpt_inter,w		; Flamme 4
	subwf	cpt_flam41,w	; cpt_flam42 = Valeur de la puissance de la flamme 4 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf 	PORTB,7			; Allume la flamme 4 PWM.
	btfss	STATUS,C
	bcf 	PORTB,7			; Eteint la flamme 4 PWM.

pas_flammes	
	movlw   200				; Dure = (255-XXX)*temps_us du Timer0 (=112S/2S) = 56.   255-57 = 200.
	movwf   TMR0			; Interruption toutes les 112s*2 = 224 instructions  partir d'ici, soit 120 S avec le code de l'interruption.
	bcf     INTCON,T0IF		; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS			; restaurer status
	swapf   w_temp,f		; Inversion L et H de l'ancien W sans modifier Z
	swapf   w_temp,w  		; Rinversion de L et H dans W. W restaur sans modifier status
	retfie  				; Retour d'interruption.
;
;********************************************************************************************
;	Squence d'inititialisation du PIC ici.
; 	Suite de cette squence en 0x900, pour ne pas encombrer la plage en BANK0 de 0x000  x0x4FF.
;
init	
	bsf 	PCLATH,3	; Prpare le saut vers BANK1 (0x800 - 0xFFF).
	call	init_suite	; Saut en BANK1.
	bcf 	PCLATH,3	; On reste ensuite toujours en BANK0 (0x000 - 0x7FF) pour les CALL !
	movlw   200			; Dure = (255-XXX)*temps_us du Timer0 (=112S/2S) = 56.   255-56 = 200.
	movwf   TMR0		; Interruption toutes les 112s*2 = 224 instructions  partir d'ici.
	bcf     INTCON,T0IF	; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.
	bsf     INTCON,GIE	; L'initialisation est finie, on autorise toutes les interruptions.
;
; Boucle principale
; --------------------------------------------------------------------------------------------
; On restera en BANK0, on ne passe plus en BANK1 ou BANK2 dans cette boucle principale.
; --------------------------------------------------------------------------------------------
;
boucle
	btfss	status_fluo,6	; = 1 ?
	goto	boucle
	bcf		status_fluo,6	; On passe ici toutes les 120S. On vient de finir le programme d'interruption. On remet le bit(0)  0.
; --------------------------------------------------------------------------------------------
; Comme le sous-programem "Flammes" est trs gros, on y saute directement pour ne traiter que celui-ci.	
	btfss	mode_fonc2,1	; bit(1) =1/0 => Flammes = ON/OFF
	goto	flam_test_001	
	btfsc	status_fluo,7	; bit(1) =1 => On vient de finir la squence PMW des flammes, on peut recalculer leurs valeurs.
	goto	flam_debut		; On passera dans les autres sous-programmes dans 120 s.
flam_test_001	
;********************************************************************************************************;
; Calcul des valeurs alatoires sur 16 bits pour les squences lumineuses.
; Initialiser les registres 'varale1s' et 'varale2s' avec une valeur diffrente de zro sur 16 bits.
; Source web : https://www.dontronics.com/psbpix/random.html
;            : http://www.joescat.com/micro/pic/an544_rng.html
; 16bit Pseudo Squence Random Number Generator (RNG).
; Based on Microchip's MATH routine found in AN544 and modified to fit all picchips including the tiny 12bit versions.
; This version uses 10 program words, making it slightly faster, but it needs a temp scratch pad to work.
; This RNG is based on linear shift register feedback. Sequence is generated by (Q15 xor Q14 xor Q12 xor Q3).
; Ne pas modifier les variables 'varale1/2s' et 'varale1/2s+1', autre part dans le programme.
; Utiliser les variables 'varale1s' et 'varale1s+1' sans les modifier.
;
	decfsz	cpt_vara01,f		; On passe ici toutes les 120 s.
	goto	valea_fin
	movf	varale1s,w		; On passe ici toutes les 31 ms. First, ensure that hiB and lowB aren't
	iorwf	varale1s+1,w	; all zeros. If they are, NOT hiB to FFh.
	btfsc	STATUS,Z		; Otherwise, leave hiB and lowB as is.
	comf	varale1s,f
	movlw	0x80			; We want to XOR hiB.7, hiB.6, hiB.4
	btfsc	varale1s,6		; and lowB.3 together in W. Rather than
	xorwf	varale1s,f		; try to line up these bits, we just
	btfsc	varale1s,4		; check to see whether a bit is a 1. If it
	xorwf	varale1s,f		; is, XOR 80h into hiB. If it isn't,
	btfsc	varale1s+1,3	; do nothing. When we're done, the
	xorwf	varale1s,f		; XOR of the 4 bits will be in hiB.7.
	rlf 	varale1s,w		; Move hiB.7 into carry.
	rlf 	varale1s+1,f	; Rotate c into lowB.0, lowB.7 into c.
	rlf 	varale1s,f		; Rotate c into hiB.0.
	movf	varale1s+1,w
	xorwf	varale1s,w		; 'w' = 'varale1s' + 'varale1s+1'
	movwf	varale1s+2
	movf	varale2s,w		; First, ensure that hiB and lowB aren't
	iorwf	varale2s+1,w	; all zeros. If they are, NOT hiB to FFh.
	btfsc	STATUS,Z		; Otherwise, leave hiB and lowB as is.
	comf	varale2s,f
	movlw	0x80			; We want to XOR hiB.7, hiB.6, hiB.4
	btfsc	varale2s,6		; and lowB.3 together in W. Rather than
	xorwf	varale2s,f		; try to line up these bits, we just
	btfsc	varale2s,4		; check to see whether a bit is a 1. If it
	xorwf	varale2s,f		; is, XOR 80h into hiB. If it isn't,
	btfsc	varale2s+1,3	; do nothing. When we're done, the
	xorwf	varale2s,f		; XOR of the 4 bits will be in hiB.7.
	rlf 	varale2s,w		; Move hiB.7 into carry.
	rlf 	varale2s+1,f	; Rotate c into lowB.0, lowB.7 into c.
	rlf 	varale2s,f		; Rotate c into hiB.0.
	movf	varale2s+1,w
	xorwf	varale2s,w		; 'w' = 'varale2s' + 'varale2s+1'
	movwf	varale2s+2
valea_fin

; -------------------------------------------------------------------------------
; Feux de pntration.
;   cpt_feup11	; Compteur de boucle principale.
;   cpt_feup12	; Compteur de boucle secondaire.
;   cpt_feup13	; Compteur ralentisseur de boucle.
;   cpt_feup14	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash, 4=> Pause longue dure
;   cpt_feup15	; Compteur de boucle en pause.
;   cpt_feup16  ; 2, 4, 6, 8, 10 => Squence 1 / 2 / 3 / 4 / 5 flashs de suite.
; -------------------------------------------------------------------------------
;
feup_debut
feup1_debut
	btfss	mode_fonc2,2	; bit(2) =1/0 => Feux pntration 1 & 2 = ON/OFF
	goto	feup1_fin
	btfsc	status_pha,0	; Etat = 00 ?
	goto	feup1_001
	btfsc	status_pha,1
	goto	feup1_001
; Phares et feux pntrants teints en mode pause longue dure.
	decfsz	cpt_feup11,f	; On passe ici toutes les 120 s.
	goto	feup1_fin
	decfsz	cpt_feup12,f	; On passe ici toutes les 31 msec = 256*120 s.
	goto	feup1_fin
	movlw	32
	movwf	cpt_feup12		; Pour une boucle de 1 sec.
	decfsz	cpt_feup13,f	; On passe ici toutes les 1 sec = 32*256*120 s.
	goto	feup1_fin		
; Pour la pause longue = cpt_feup13 = 7  23 = 1 mn  3 mn. 
	bsf		status_pha,0	; Etat = 01 !
	btfsc	mode_fonc2,7	; bit(7) =1/0 => Phares 1 & 2 = ON/OFF
	bsf		PORTA,5			; Mise en marche du phare 1.
	movlw	7
	movwf	cpt_feup12		; Boucle de 0,2 sec.
	movf	varale1s,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	5				; (5  36)*0,2 sec = 1  7 secondes
	movwf	cpt_feup13		; Temps d'attente entre la mise en route des phares et des feux pntrants.
	goto	feup1_fin	
feup1_001
	btfss	status_pha,0	; Etat = 01 ?
	goto	feup1_002
	btfsc	status_pha,1
	goto	feup1_002
; Phares allums en attente d'allumage des feux pntrants.
	decfsz	cpt_feup11,f	; On passe ici toutes les 120 s.
	goto	feup1_fin
	decfsz	cpt_feup12,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	feup1_fin
	movlw	7
	movwf	cpt_feup12		; Pour une boucle de 0,2 sec.
	decfsz	cpt_feup13,f	; On passe ici toutes les 0,2 sec
	goto	feup1_fin		
	bcf		status_pha,0	; Etat = 10 !
	bsf		status_pha,1
	movf	varale2s,w		; Dlai alatoire de marche.
	andlw	31
	addlw	2*PEN_DF1		; (0  31) + 2*PEN_DF1(5  110) secondes = 5  125 sec.
	movwf	cpt_feup15
	goto	feup1_fin
feup1_002
	btfsc	status_pha,0	; Etat = 10 ?
	goto	feup1_004
	btfss	status_pha,1
	goto	feup1_004
; Phares et feux pntrants allums.
	decfsz	cpt_feup11,f	
	goto	feup1_fin
	movlw	10
	movwf	cpt_feup11		; Pour ralentir le passage dans la  boucle suivante.
; On passe ici toutes les 1,2 msec.
	decfsz	cpt_feup12,f
	goto	feup1_fin
; On passe ici toutes les 2, 70 ou 250 msec.
	btfsc	cpt_feup14,0	
	goto	feup1_003	
; Allume un feu pntrant, cpt_feup14(0) = 0 !
	btfss	cpt_feup15,0	
	bsf 	PORTC,3			; Allume le feu D, cpt_feup14 = 0,2,4,6,8,10...
	btfsc	cpt_feup15,0	
	bsf 	PORTC,6			; Allume le feu G, cpt_feup14 = 0,2,4,6,8,10...
	movlw	2				; Prochaine pause de 2 msec.
	movwf	cpt_feup12	
	incf	cpt_feup14,f
	goto	feup1_fin
feup1_003	
; Eteint un feu pntrant, cpt_feup14(0) = 1 !
	btfss	cpt_feup15,0	
	bcf 	PORTC,3			; Eteint le feu D, cpt_feup14 = 1,3,5,7,11...
	btfsc	cpt_feup15,0	
	bcf 	PORTC,6			; Eteint le feu G, cpt_feup14 = 1,3,5,7,11...
	movlw	70				; Prochaine pause de 70 msec avant le prochain allumage.
	movwf	cpt_feup12	
	incf	cpt_feup14,f
	movf	cpt_feup14,w
	subwf	cpt_feup16,w	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	btfss	STATUS,Z		; Sequence n 8 ?
	goto	feup1_fin
	movlw	253				; Prochaine pause de 250 msec avant de passer  l'autre feu.
	movwf	cpt_feup12	
	clrf	cpt_feup14		; Raz compteur de squence de flash.
	decfsz	cpt_feup15,f
	goto	feup1_fin
	bsf		status_pha,0	; Etat = 11 !
	movlw	7
	movwf	cpt_feup12		; Boucle de 0,2 sec.
	movf	varale1s,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	5				; (5  36)*0,2 sec = 1  7 secondes
	movwf	cpt_feup13		; Temps d'attente entre l'arrt des feux pntrants et des phares.
	goto	feup1_fin	
feup1_004
	btfss	status_pha,0	; Etat = 11 ?
	goto	feup1_fin
	btfss	status_pha,1
	goto	feup1_fin
; Phares allums en attente d'extinction.
	decfsz	cpt_feup11,f	; On passe ici toutes les 120 s.
	goto	feup1_fin
	decfsz	cpt_feup12,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	feup1_fin
	movlw	7
	movwf	cpt_feup12		; Pour une boucle de 0,2 sec.
	decfsz	cpt_feup13,f	; On passe ici toutes les 0,2 sec
	goto	feup1_fin		
	bcf		status_pha,0	; Etat = 00 !
	bcf		status_pha,1
	btfsc	mode_fonc2,7	; bit(7) =1/0 => Phares 1 & 2 = ON/OFF
	bcf		PORTA,5			; Arrt du phare 1.
	movlw	32
	movwf	cpt_feup12		; Pour une boucle de 1 sec.
	movf	varale2s,w		; Dlai alatoire d'arrt
	andlw	31
	addlw	PEN_PO1			; (0  31) + PEN_PO1(5  220) secondes = 5  251 sec.
	movwf	cpt_feup13
feup1_fin	
;---------------------------------------------------------
feup2_debut
	btfss	mode_fonc2,2	; bit(2) =1/0 => Feux pntration 1 & 2 = ON/OFF
	goto	feup2_fin
	btfsc	status_pha,2	; Etat = 00xx ?
	goto	feup2_001
	btfsc	status_pha,3
	goto	feup2_001
; Phares et feux pntrants teints en mode pause longue dure.
	decfsz	cpt_feup21,f	; On passe ici toutes les 120 s.
	goto	feup2_fin
	decfsz	cpt_feup22,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	feup2_fin
	movlw	33
	movwf	cpt_feup22		; Pour une boucle de 1 sec.
	decfsz	cpt_feup23,f	; On passe ici toutes les 1 sec = 32*256*120 s.
	goto	feup2_fin		
; Pour la pause longue = cpt_feup23 = 7  23 = 1 mn  3 mn. 
	bsf		status_pha,2	; Etat = 01xx !
	btfsc	mode_fonc2,7	; bit(7) =1/0 => Phares 1 & 2 = ON/OFF
	bsf		PORTA,4			; Mis en marche du phare 2.
	movlw	7
	movwf	cpt_feup22		; Boucle de 0,2 sec.
	movf	varale1s+1,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	6				; (6  37)*0,2 sec = 1  7 secondes
	movwf	cpt_feup23		; Temps d'attente entre la mise en route des phares et des feux pntrants.
	goto	feup2_fin	
feup2_001
	btfss	status_pha,2	; Etat = 01xx ?
	goto	feup2_002
	btfsc	status_pha,3
	goto	feup2_002
; Phares allums en attente d'allumage des feux pntrants.
	decfsz	cpt_feup21,f	; On passe ici toutes les 120 s.
	goto	feup2_fin
	decfsz	cpt_feup22,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	feup2_fin
	movlw	7
	movwf	cpt_feup22		; Pour une boucle de 0,2 sec.
	decfsz	cpt_feup23,f	; On passe ici toutes les 0,2 sec
	goto	feup2_fin		
	bcf		status_pha,2	; Etat = 10xx !
	bsf		status_pha,3
	movf	varale2s+1,w		; Dlai alatoire de marche.
	andlw	31
	addlw	2*PEN_DF2		; (0  31) + 2*PEN_DF2(5  110) secondes = 5  125 sec.
	movwf	cpt_feup25
	goto	feup2_fin
feup2_002
	btfsc	status_pha,2	; Etat = 10xx ?
	goto	feup2_004
	btfss	status_pha,3
	goto	feup2_004
; Phares et feux pntrants allums.
	decfsz	cpt_feup21,f	
	goto	feup2_fin
	movlw	11				; 11#10 = Lger dcalage avec la voiture 1.
	movwf	cpt_feup21		; Pour ralentir le passage dans la  boucle suivante.
; On passe ici toutes les 1,3 msec.
	decfsz	cpt_feup22,f
	goto	feup2_fin
; On passe ici toutes les 2, 70 ou 250 msec.
	btfsc	cpt_feup24,0	
	goto	feup2_003	
; Allume un feu pntrant, cpt_feup24(0) = 0 !
	btfss	cpt_feup25,0	
	bsf 	PORTC,7			; Allume le feu D, cpt_feup24 = 0,2,4,6,8,10...
	btfsc	cpt_feup25,0	
	bsf 	PORTB,7			; Allume le feu G, cpt_feup24 = 0,2,4,6,8,10...
	movlw	2				; Prochaine pause de 2 msec.
	movwf	cpt_feup22	
	incf	cpt_feup24,f
	goto	feup2_fin
feup2_003	
; Eteint un feu penetrant, cpt_feup24(0) = 1 !
	btfss	cpt_feup25,0	
	bcf 	PORTC,7			; Eteint le feu D, cpt_feup24 = 1,3,5,7,11...
	btfsc	cpt_feup25,0	
	bcf 	PORTB,7			; Eteint le feu G, cpt_feup24 = 1,3,5,7,11...
	movlw	72				; Prochaine pause de 72 msec avant le prochain allumage.
	movwf	cpt_feup22	
	incf	cpt_feup24,f
	movf	cpt_feup24,w
	subwf	cpt_feup26,w	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	btfss	STATUS,Z		; Sequence n 8 ?
	goto	feup2_fin
	movlw	253				; Prochaine pause de 250 msec avant de passer  l'autre feu.
	movwf	cpt_feup22	
	clrf	cpt_feup24		; Raz compteur de squence de flash.
	decfsz	cpt_feup25,f
	goto	feup2_fin
	bsf		status_pha,2	; Etat = 11xx !
	movlw	7
	movwf	cpt_feup22		; Boucle de 0,2 sec.
	movf	varale1s+1,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	5				; (5  36)*0,2 sec = 1  7 secondes
	movwf	cpt_feup23		; Temps d'attente entre l'arrt des feux pntrants et des phares.
	goto	feup2_fin	
feup2_004
	btfss	status_pha,2	; Etat = 11xx ?
	goto	feup2_fin
	btfss	status_pha,3
	goto	feup2_fin
; Phares allums en attente d'extinction.
	decfsz	cpt_feup21,f	; On passe ici toutes les 120 s.
	goto	feup2_fin
	decfsz	cpt_feup22,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	feup2_fin
	movlw	7
	movwf	cpt_feup22		; Pour une boucle de 0,2 sec.
	decfsz	cpt_feup23,f	; On passe ici toutes les 0,2 sec
	goto	feup2_fin		
	bcf		status_pha,2	; Etat = 00xx !
	bcf		status_pha,3
	btfsc	mode_fonc2,7	; bit(7) =1/0 => Phares 1 & 2 = ON/OFF
	bcf		PORTA,4			; Arrt du phare 2.
	movlw	32
	movwf	cpt_feup22		; Pour une boucle de 1 sec.
	movf	varale2s+1,w		; Dlai alatoire d'arrt
	andlw	31
	addlw	PEN_PO2			; (0  31) + PEN_PO2(5  220) secondes = 5  251 sec.
	movwf	cpt_feup23
feup2_fin	

arc_debut
; -------------------------------------------------------
; Squence arc lectrique 1.
arc1_debut
	btfss	mode_fonc2,0	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
	goto	arc1_fin
	btfsc	cpt_arc18,0		; Arc 1 actif/inactif longue dure ?
	goto	arc1_001
	decfsz	cpt_arc11,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	arc1_fin
	decfsz	cpt_arc12,f		; On passe ici toutes les 31 ms (256*120 s).
	goto	arc1_fin
	movlw	32
	movwf	cpt_arc12
	decfsz	cpt_arc13,f		; On passe ici toutes les secondes.
	goto	arc1_fin
	movlw	1
	movwf	cpt_arc14
	movwf	cpt_arc15
	movlw	ARC_DF1		; 30 = temps actif => 30 secondes actif.
	movwf	cpt_arc13	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc18	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf		cpt_arc18,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	arc1_fin	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).
arc1_001	
	decfsz	cpt_arc11,f ; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	arc1_002
	decfsz	cpt_arc12,f	; On passe ici toutes les 30 ms (256*120 s).
	goto	arc1_002
	movlw	32
	movwf	cpt_arc12
	decfsz	cpt_arc13,f	; On passe ici toutes les secondes.
	goto	arc1_002
	movlw	ARC_PO1		; 40 = Temps de pause => 40 secondes inactif.
	movwf	cpt_arc13	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf		PORTA,1		; On teint le flash.
	bcf		cpt_arc18,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	arc1_fin	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).
arc1_002	
	btfss	cpt_arc18,1
	goto	arc1_005
	btfsc	cpt_arc18,2
	goto	arc1_004	
	decfsz	cpt_arc16,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	arc1_fin
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc17,f
	goto	arc1_fin
	decfsz	cpt_arc14,f	; Fin de courte squence de flash allum.
	goto	arc1_003
	movf	varale1s,w	; On a fini les squences de flash.	
	andlw	0x1F		; ARC1
	addlw	16			; 16-47 : ARC1 pour 0,4  1,4 secondes de pause courte dure.
;;;	andlw	0x1F		; ARC2
;;;	addlw	8			;  8-39 : ARC2 pour 0,3  1,3 secondes de pause courte dure.
;;;	andlw	0x1F		; ARC3
;;;	addlw	4			;  4-35 : ARC3 pour 0,1  1,1 secondes de pause courte dure
	movwf	cpt_arc15 	
	bcf		cpt_arc18,1	; Bit(1) = 0. 
	goto	arc1_fin	; On change de mode => Flash en arrt courte dure (cpt_arc_14, 15).
arc1_003	
	movf	varale2s,w
	andlw	0x3F		
	movwf	cpt_arc17	; 0-63
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addlw	10			; 10-41
	addwf	cpt_arc17,f	; 10-104  cpt_arc17 = 10  104 ms.
	bsf		PORTA,1		; On allume le flash.
	bsf		cpt_arc18,2	; Bit(2) = 1. Flash allume.
	goto	arc1_fin	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc17).
arc1_004	
	decfsz	cpt_arc16,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	arc1_fin	
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc17,f
	goto	arc1_fin	
	movwf	cpt_arc17	; cpt_arc17 = 10  101 ms.
	movf	varale2s+1,w
	andlw	0x0F		
	movwf	cpt_arc17	; 0-15
	movf	varale1s,w
	andlw	0X07		; 0-7
	addlw	10			; 10-17
	addwf	cpt_arc17,f	; 10-32  cpt_arc17 = 10  32 ms.
	bcf		PORTA,1		; on teint le flash.
	bcf		cpt_arc18,2	; Bit(2) = 0. Flash teint.
	goto	arc1_fin	; On change de mode => Flash teint (cpt_arc_16,17).
arc1_005	
	decfsz	cpt_arc14,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	arc1_fin
	decfsz	cpt_arc15,f	; On passe ici toutes les 30 msec.
	goto	arc1_fin
	movf	varale2s,w	; On passe ici apres une courte pause de 1,5  8 secondes.
	andlw	0x1F		; ARC1
	addlw	10			;  ARC1 : 10  41 squences lumineuses de suite.
;;;	andlw	0x0F		; ARC2
;;;	addlw	5			;  ARC2 :  5  20 squences lumineuses de suite.
;;;	andlw	0x07		; ARC3
;;;	addlw	3			;  ARC3 :  3  10 squences lumineuses de suite.
	movwf	cpt_arc14
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc17	; cpt_arc17 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf		PORTA,1		; On allume le flash.
	bsf		cpt_arc18,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf		cpt_arc18,2	; Bit(2)=1 =>  Flash allum.
	goto	arc1_fin	; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).
arc1_fin
;---------------------------------------------------------
arc2_debut
	btfss	mode_fonc2,0	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
	goto	arc2_fin
	btfsc	cpt_arc28,0		; Arc 1 actif/inactif longue dure ?
	goto	arc2_001
	decfsz	cpt_arc21,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	arc2_fin
	decfsz	cpt_arc22,f		; On passe ici toutes les 30 ms (256*120 s).
	goto	arc2_fin
	movlw	32
	movwf	cpt_arc22
	decfsz	cpt_arc23,f		; On passe ici toutes les secondes.
	goto	arc2_fin
	movlw	1
	movwf	cpt_arc24
	movwf	cpt_arc25
	movlw	ARC_DF2		; 30 = temps actif => 30 secondes actif.
	movwf	cpt_arc23	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc28	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf		cpt_arc28,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	arc2_fin	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).
arc2_001	
	decfsz	cpt_arc21,f ; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	arc2_002
	decfsz	cpt_arc22,f	; On passe ici toutes les 30 ms (256*120 s).
	goto	arc2_002
	movlw	32
	movwf	cpt_arc22
	decfsz	cpt_arc23,f	; On passe ici toutes les secondes.
	goto	arc2_002
	movlw	ARC_PO2		; 40 = Temps de pause => 40 secondes inactif.
	movwf	cpt_arc23	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf		PORTA,2		; On teint le flash.
	bcf		cpt_arc28,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	arc2_fin	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).
arc2_002	
	btfss	cpt_arc28,1
	goto	arc2_005
	btfsc	cpt_arc28,2
	goto	arc2_004	
	decfsz	cpt_arc26,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	arc2_fin
	movlw	9
	movwf	cpt_arc26	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc27,f
	goto	arc2_fin
	decfsz	cpt_arc24,f	; Fin de courte squence de flash allum.
	goto	arc2_003
	movf	varale1s+1,w	; On a fini les squences de flash.	
;;;	andlw	0x1F		; ARC1
;;;	addlw	16			; 16-47 : ARC1 pour 0,4  1,4 secondes de pause courte dure.
	andlw	0x1F		; ARC2
	addlw	8			;  8-39 : ARC2 pour 0,3  1,3 secondes de pause courte dure.
;;;	andlw	0x1F		; ARC3
;;;	addlw	4			;  4-35 : ARC3 pour 0,1  1,1 secondes de pause courte dure
	movwf	cpt_arc25
	bcf		cpt_arc28,1	; Bit(1) = 0. 
	goto	arc2_fin	; On change de mode => Flash en arrt courte dure (cpt_arc_14, 15).
arc2_003	
	movf	varale2s+1,w
	andlw	0x3F		
	movwf	cpt_arc27	; 0-63
	movf	varale1s,w
	andlw	0x1F		; 0-31
	addlw	10			; 10-41
	addwf	cpt_arc27,f	; 10-104  cpt_arc27 = 10  104 ms.
	bsf		PORTA,2		; On allume le flash.
	bsf		cpt_arc28,2	; Bit(2) = 1. Flash allume.
	goto	arc2_fin	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc27).
arc2_004	
	decfsz	cpt_arc26,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	arc2_fin	
	movlw	9
	movwf	cpt_arc26	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc27,f
	goto	arc2_fin	
	movwf	cpt_arc27	; cpt_arc27 = 10  101 ms.
	movf	varale2s,w
	andlw	0x0F		
	movwf	cpt_arc27	; 0-15
	movf	varale2s+1,w
	andlw	0X07		; 0-7
	addlw	10			; 10-17
	addwf	cpt_arc27,f	; 10-32  cpt_arc27 = 10  32 ms.
	bcf		PORTA,2		; on teint le flash.
	bcf		cpt_arc28,2	; Bit(2) = 0. Flash teint.
	goto	arc2_fin	; On change de mode => Flash teint (cpt_arc_16,17).
arc2_005	
	decfsz	cpt_arc24,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	arc2_fin
	decfsz	cpt_arc25,f	; On passe ici toutes les 30 msec.
	goto	arc2_fin
	movf	varale2s,w	; On passe ici apres une courte pause de 1,5  8 secondes.
;;;	andlw	0x1F		; ARC1
;;;	addlw	10			;  ARC1 : 10  41 squences lumineuses de suite.
	andlw	0x0F		; ARC2
	addlw	5			;  ARC2 :  5  20 squences lumineuses de suite.
;;;	andlw	0x07		; ARC3
;;;	addlw	3			;  ARC3 :  3  10 squences lumineuses de suite.
	movwf	cpt_arc24
	movlw	9
	movwf	cpt_arc26	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc27	; cpt_arc27 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf		PORTA,2		; On allume le flash.
	bsf		cpt_arc28,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf		cpt_arc28,2	; Bit(2)=1 =>  Flash allum.
	goto	arc2_fin	; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).
arc2_fin
;---------------------------------------------------------
; -------------------------------------------------------
; Squence arc lectrique 1.
arc3_debut
	btfss	mode_fonc2,5	; bit(5) =1/0 => Arc 3  = ON/OFF
	goto	arc3_fin
	btfsc	cpt_arc38,0		; Arc 1 actif/inactif longue dure ?
	goto	arc3_001
	decfsz	cpt_arc31,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	arc3_fin
	decfsz	cpt_arc32,f		; On passe ici toutes les 30 ms (256*120 s).
	goto	arc3_fin
	movlw	32
	movwf	cpt_arc32
	decfsz	cpt_arc33,f		; On passe ici toutes les secondes.
	goto	arc3_fin
	movlw	1
	movwf	cpt_arc34
	movwf	cpt_arc35
	movlw	ARC_DF3		; 30 = temps actif => 30 secondes actif.
	movwf	cpt_arc33	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc38	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf		cpt_arc38,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	arc3_fin	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).
arc3_001	
	decfsz	cpt_arc31,f ; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	arc3_002
	decfsz	cpt_arc32,f	; On passe ici toutes les 30 ms (256*120 s).
	goto	arc3_002
	movlw	32
	movwf	cpt_arc32
	decfsz	cpt_arc33,f	; On passe ici toutes les secondes.
	goto	arc3_002
	movlw	ARC_PO3		; 40 = Temps de pause => 40 secondes inactif.
	movwf	cpt_arc33	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf		PORTC,0		; On teint le flash.
	bcf		cpt_arc38,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	arc3_fin	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).
arc3_002	
	btfss	cpt_arc38,1
	goto	arc3_005
	btfsc	cpt_arc38,2
	goto	arc3_004	
	decfsz	cpt_arc36,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	arc3_fin
	movlw	9
	movwf	cpt_arc36	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc37,f
	goto	arc3_fin
	decfsz	cpt_arc34,f	; Fin de courte squence de flash allum.
	goto	arc3_003
	movf	varale1s,w	; On a fini les squences de flash.	
;;;	andlw	0x1F		; ARC1
;;;	addlw	16			; 16-47 : ARC1 pour 0,4  1,4 secondes de pause courte dure.
;;;	andlw	0x1F		; ARC2
;;;	addlw	8			;  8-39 : ARC2 pour 0,3  1,3 secondes de pause courte dure.
	andlw	0x1F		; ARC3
	addlw	4			;  4-35 : ARC3 pour 0,1  1,1 secondes de pause courte dure
	movwf	cpt_arc35
	bcf		cpt_arc38,1	; Bit(1) = 0. 
	goto	arc3_fin	; On change de mode => Flash en arrt courte dure (cpt_arc_14, 15).
arc3_003	
	movf	varale2s+1,w
	andlw	0x3F		
	movwf	cpt_arc37	; 0-63
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addlw	10			; 10-41
	addwf	cpt_arc37,f	; 10-104  cpt_arc37 = 10  104 ms.
	bsf		PORTC,0		; On allume le flash.
	bsf		cpt_arc38,2	; Bit(2) = 1. Flash allume.
	goto	arc3_fin	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc37).
arc3_004	
	decfsz	cpt_arc36,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	arc3_fin	
	movlw	9
	movwf	cpt_arc36	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc37,f
	goto	arc3_fin	
	movwf	cpt_arc37	; cpt_arc37 = 10  101 ms.
	movf	varale2s,w
	andlw	0x0F		
	movwf	cpt_arc37	; 0-15
	movf	varale1s,w
	andlw	0X07		; 0-7
	addlw	10			; 10-17
	addwf	cpt_arc37,f	; 10-32  cpt_arc37 = 10  32 ms.
	bcf		PORTC,0		; on teint le flash.
	bcf		cpt_arc38,2	; Bit(2) = 0. Flash teint.
	goto	arc3_fin	; On change de mode => Flash teint (cpt_arc_16,17).
arc3_005	
	decfsz	cpt_arc34,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	arc3_fin
	decfsz	cpt_arc35,f	; On passe ici toutes les 30 msec.
	goto	arc3_fin
	movf	varale2s,w	; On passe ici apres une courte pause de 1,5  8 secondes.
;;;	andlw	0x1F		; ARC1
;;;	addlw	10			;  ARC1 : 10  41 squences lumineuses de suite.
;;;	andlw	0x0F		; ARC2
;;;	addlw	5			;  ARC2 :  5  20 squences lumineuses de suite.
	andlw	0x07		; ARC3
	addlw	3			;  ARC3 :  3  10 squences lumineuses de suite.
	movwf	cpt_arc34
	movlw	9
	movwf	cpt_arc36	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc37	; cpt_arc37 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf		PORTC,0		; On allume le flash.
	bsf		cpt_arc38,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf		cpt_arc38,2	; Bit(2)=1 =>  Flash allum.
	goto	arc3_fin	; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).
arc3_fin
arc_fin

; -------------------------------------------------------
; Squence tubes fluorescents.
; cpt_fluo11	; 1er  Compteur de temps actif/inactif.
; cpt_fluo12	; 2me Compteur de temps actif/inactif.
; cpt_fluo13	; 3me Compteur de temps actif/inactif.
; cpt_fluo14	; Compteur de squence lumineuse pour tube fluorescent. 
; status_fluo	; Etat gnral de fonctionnement du tube.
; 		; bit(0)=0 => Tube en fonctionnement longue dure
; 		; bit(0)=1 => Tube en pause longue dure
; Des temps d'attente lgrement diffrents dans cpt_fluox2,
; pour viter des allumages synchrones.
;
; Tube Fluo 1 = Cinq tentatives d'allumage puis reste allum un temps relativement long.
; Tube Fluo 2 = Nombreuses tentatives d'allumage, mais ne reste allum qu'un court moment.
; Tube Fluo 3 = Tentatives d'allumage en continu, mais ne parvient pas  s'allumer.
; Squence tube fluorescent 1.
fluo1_debut
	btfss	mode_fonc2,0	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
	goto	fluo1_fin
	btfss	status_fluo,0	; Tube fluorescent en pose longue dure ?
	goto	fluo1_001
	decfsz	cpt_fluo11,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	fluo1_fin
	decfsz	cpt_fluo12,f	; On passe ici toutes les 31 ms (256*120 s).
	goto	fluo1_fin
	movlw	31				; FLUO 1
;;;	movlw	32 				; FLUO 2
;;;	movlw	33				; FLUO 3
	movwf	cpt_fluo12
	decfsz	cpt_fluo13,f	; On passe ici toutes les secondes.
	goto	fluo1_fin		; Le temps de pause = cpt_fluox3 (Secondes).
	bcf		status_fluo,0	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo12
	clrf	cpt_fluo14		; Raz du compteur de squence lumineuse.
	goto	fluo1_fin
fluo1_001	
	decfsz	cpt_fluo11,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	fluo1_fin
	decfsz	cpt_fluo12,f	; On passe ici toutes les 31 ms (256*120 s).
	goto	fluo1_fin
	movlw	VIT_FL1			; 7,8,9 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo12		; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo14,w	; cpt_fluo14 = Compteur de squence de tube fluorescent.
	call	table_fluo		; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,0			; Bit(0) = Tube fluorescent 1.
	bcf		PORTC,1
	btfsc	temp1,0
	bsf		PORTC,1
	incf	cpt_fluo14,f	; cpt_fluo14 = Compteur de squence du tube fluo.
	btfss	temp1,7			; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	fluo1_fin
	bsf		status_fluo,0	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL1			; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo13
fluo1_fin
; -------------------------------------------------------	
fluo2_debut
	btfss	mode_fonc2,0	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
	goto	fluo2_fin
	btfss	status_fluo,1	; Tube fluorescent 1 en pose longue dure ?
	goto	fluo2_001
	decfsz	cpt_fluo21,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	fluo2_fin
	decfsz	cpt_fluo22,f	; On passe ici toutes les 19 ms (128*165 s).
	goto	fluo2_fin
;;;	movlw	31				; FLUO 1
	movlw	32 				; FLUO 2
;;;	movlw	33				; FLUO 3
	movwf	cpt_fluo22
	decfsz	cpt_fluo23,f	; On passe ici toutes les secondes.
	goto	fluo2_fin		; Le temps de pause = cpt_fluox3 (Secondes).
	bcf		status_fluo,1	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo22
	clrf	cpt_fluo24		; Raz du compteur de squence lumineuse.
	goto	fluo2_fin
fluo2_001	
	decfsz	cpt_fluo21,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	fluo2_fin
	decfsz	cpt_fluo22,f	; On passe ici toutes les 19 ms (127*165 s).
	goto	fluo2_fin
	movlw	VIT_FL2			; 9 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo22		; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo24,w	; cpt_fluo24 = Compteur de squence de tube fluorescent.
	call	table_fluo		; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,1			; bit(1) = Tube fluorescent 2.
	bcf		PORTC,2
	btfsc	temp1,1
	bsf		PORTC,2
	incf	cpt_fluo24,f	; cpt_fluo24 = Compteur de squence du tube fluo.
	btfss	temp1,7			; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	fluo2_fin
	bsf		status_fluo,1	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL2			; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo23
fluo2_fin
; -------------------------------------------------------	
fluo3_debut
	btfss	mode_fonc2,6	; ; bit(6) =1/0 => Fluo 3 = ON/OFF
	goto	fluo3_fin
	btfss	status_fluo,2	; Tube fluorescent 3 en pose longue dure ?
	goto	fluo3_001
	decfsz	cpt_fluo31,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	fluo3_fin
	decfsz	cpt_fluo32,f	; On passe ici toutes les 19 ms (128*165 s).
	goto	fluo3_fin
;;;	movlw	31				; FLUO 1
;;;	movlw	32 				; FLUO 2
	movlw	33				; FLUO 3
	movwf	cpt_fluo32
	decfsz	cpt_fluo33,f	; On passe ici toutes les secondes.
	goto	fluo3_fin		; Le temps de pause = cpt_fluox3 (Secondes).
	bcf		status_fluo,2	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo32
	clrf	cpt_fluo34		; Raz du compteur de squence lumineuse.
	goto	fluo3_fin
fluo3_001	
	decfsz	cpt_fluo31,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	fluo3_fin
	decfsz	cpt_fluo32,f	; On passe ici toutes les 19 ms (127*165 s).
	goto	fluo3_fin
	movlw	VIT_FL3			; 9 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo32		; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo34,w	; cpt_fluo34 = Compteur de squence de tube fluorescent.
	call	table_fluo		; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,2			; bit(2) = Tube fluorescent 3.
	bcf		PORTB,4
	btfsc	temp1,2
	bsf		PORTB,4
	incf	cpt_fluo34,f	; cpt_fluo34 = Compteur de squence du tube fluo.
	btfss	temp1,7			; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	fluo3_fin
	bsf		status_fluo,2	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL3			; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo33
fluo3_fin
; ---------------------------------------------------------------------------------------------------
;   Gyrophare clignotant 
;
clig3_debut
	btfss	mode_fonc2,3	; bit(3) =1/0 => Gyrophare clignotant 3 & phares 3 = ON/OFF
	goto	clig3_fin
	btfsc	status_pha,4	; Etat = 00xxxx ?
	goto	clig3_001
	btfsc	status_pha,5
	goto	clig3_001
; Phares et gyrophare clignotant teints en mode pause longue dure.
	decfsz	cpt_clig31,f	; On passe ici toutes les 120 s.
	goto	clig3_fin
	decfsz	cpt_clig32,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig3_fin
	movlw	33
	movwf	cpt_clig32		; Pour une boucle de 1 sec.
	decfsz	cpt_clig33,f	; On passe ici toutes les 1 sec = 32*256*120 s.
	goto	clig3_fin		
; Pour la pause longue = cpt_clig33 = 7  23 = 1 mn  3 mn. 
	bsf		status_pha,4	; Etat = 01xxxx !
	bsf		PORTC,5			; Mis en marche des phares 3.
	movlw	7
	movwf	cpt_clig32		; Boucle de 0,2 sec.
	movf	varale1s,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	6				; (6  37)*0,2 sec = 1  7 secondes
	movwf	cpt_clig33		; Temps d'attente entre la mise en route des phares et du gyrophare clignotant.
	goto	clig3_fin	
clig3_001
	btfss	status_pha,4	; Etat = 01xxxx ?
	goto	clig3_002
	btfsc	status_pha,5
	goto	clig3_002
; Phares allums en attente d'allumage du gyrophare clignotant.
	decfsz	cpt_clig31,f	; On passe ici toutes les 120 s.
	goto	clig3_fin
	decfsz	cpt_clig32,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig3_fin
	movlw	7
	movwf	cpt_clig32		; Pour une boucle de 0,2 sec.
	decfsz	cpt_clig33,f	; On passe ici toutes les 0,2 sec
	goto	clig3_fin		
	bcf		status_pha,4	; Etat = 10xxxx !
	bsf		status_pha,5
	movlw	32				; Prochain changement d'tat dans 320 msec.
	movwf	cpt_clig32	
	movf	varale2s,w		; Dlai alatoire de marche.
	andlw	31
	addlw	3*CLI_DC3		; (0  31) + 3*CLI_DC3(5  70) secondes = 5  80 sec.
	andlw	b'11111110'		; Nombre pair pour un cycle complet du gyrophare.
	movwf	cpt_clig33
	goto	clig3_fin
clig3_002
	btfsc	status_pha,4	; Etat = 10xxxx ?
	goto	clig3_004
	btfss	status_pha,5
	goto	clig3_004
; Phares et gyrophare clignotant allums.
	decfsz	cpt_clig31,f	
	goto	clig3_fin
	decfsz	cpt_clig32,f
	goto	clig3_fin
; On passe ici toutes les 31 msec.
	movlw	b'00100000'		; Anime le gyrophare clignotant.
	xorwf	PORTB,f			; Inverse PORTC,3
	movlw	11				; Prochain changement d'tat dans 320 msec.
	movwf	cpt_clig32	
	decfsz	cpt_clig33,f
	goto	clig3_fin
	bcf 	PORTB,5			; Eteint le gyrophare clignotant.
	bsf		status_pha,4	; Etat = 11xxxx !
	movlw	7
	movwf	cpt_clig32		; Boucle de 0,2 sec.
	movf	varale1s,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	6				; (6  37)*0,2 sec = 1  7 secondes
	movwf	cpt_clig33		; Temps d'attente entre l'arrt du gyrophare clignotant et des phares.
	goto	clig3_fin	
clig3_004
	btfss	status_pha,4	; Etat = 11xxxx ?
	goto	clig3_fin
	btfss	status_pha,5
	goto	clig3_fin
; Phares allums en attente d'extinction.
	decfsz	cpt_clig31,f	; On passe ici toutes les 120 s.
	goto	clig3_fin
	decfsz	cpt_clig32,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig3_fin
	movlw	7
	movwf	cpt_clig32		; Pour une boucle de 0,2 sec.
	decfsz	cpt_clig33,f	; On passe ici toutes les 0,2 sec
	goto	clig3_fin		
	bcf		status_pha,4	; Etat = 00xxxx !
	bcf		status_pha,5
	bcf		PORTC,5			; Arrt des phares 3.
	movlw	32
	movwf	cpt_clig32		; Pour une boucle de 1 sec.
	movf	varale2s,w		; Dlai alatoire d'arrt longue dure.
	andlw	31
	addlw	CLI_PO3			; (0  31) + CLI_PO3(5  220) secondes = 5  251 sec.
	movwf	cpt_clig33
clig3_fin	
;---------------------------------------------------------
clig4_debut
	btfss	mode_fonc2,4	; bit(4) =1/0 => Feux clignotants 4 & phares 4 = ON/OFF
	goto	clig4_fin
	btfsc	status_pha,6	; Etat = 00xxxxxx ?
	goto	clig4_001
	btfsc	status_pha,7
	goto	clig4_001
; Phares et gyrophare clignotant teints en mode pause longue dure.
	decfsz	cpt_clig41,f	; On passe ici toutes les 120 s.
	goto	clig4_fin
	decfsz	cpt_clig42,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig4_fin
	movlw	32
	movwf	cpt_clig42		; Pour une boucle de 1 sec.
	decfsz	cpt_clig43,f	; On passe ici toutes les 1 sec = 32*256*120 s.
	goto	clig4_fin		
; Pour la pause longue = cpt_clig43 = 7  23 = 1 mn  3 mn. 
	bsf		status_pha,6	; Etat = 01xxxxxx !
	bsf		PORTC,4			; Mis en marche des phares 4.
	movlw	7
	movwf	cpt_clig42		; Boucle de 0,2 sec.
	movf	varale1s+1,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	5				; (5  36)*0,2 sec = 1  7 secondes
	movwf	cpt_clig43		; Temps d'attente entre la mise en route des phares et du gyrophare clignotant.
	goto	clig4_fin	
clig4_001
	btfss	status_pha,6	; Etat = 01xxxxxx ?
	goto	clig4_002
	btfsc	status_pha,7
	goto	clig4_002
; Phares allums en attente d'allumage du gyrophare clignotant.
	decfsz	cpt_clig41,f	; On passe ici toutes les 120 s.
	goto	clig4_fin
	decfsz	cpt_clig42,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig4_fin
	movlw	7
	movwf	cpt_clig42		; Pour une boucle de 0,2 sec.
	decfsz	cpt_clig43,f	; On passe ici toutes les 0,2 sec
	goto	clig4_fin		
	bcf		status_pha,6	; Etat = 10xxxxxx !
	bsf		status_pha,7
	movlw	34				; 32#34 = Lger dcalage avec la voiture 3. Prochain changement d'tat dans 340 msec.
	movwf	cpt_clig42	
	movf	varale2s+1,w		; Dlai alatoire de marche.
	andlw	31
	addlw	3*CLI_DC4		; (0  31) + 3*CLI_DC4(5  70) secondes = 5  80 sec.
	andlw	b'11111110'		; Nombre pair pour un cycle complet du gyrophare.
	movwf	cpt_clig43
	goto	clig4_fin
clig4_002
	btfsc	status_pha,6	; Etat = 10xxxxxx ?
	goto	clig4_004
	btfss	status_pha,7
	goto	clig4_004
; Phares et gyrophare clignotant allums.
	decfsz	cpt_clig41,f	
	goto	clig4_fin
	decfsz	cpt_clig42,f
	goto	clig4_fin
; On passe ici toutes les 31 msec.
	movlw	b'01000000'		; Anime le gyrophare clignotant.
	xorwf	PORTB,f			; Inverse PORTC,7
	movlw	12				; Prochain changement d'tat dans 370 msec.
	movwf	cpt_clig42	
	decfsz	cpt_clig43,f
	goto	clig4_fin
	bcf 	PORTB,6			; Eteint le gyrophare clignotant.
	bsf		status_pha,6	; Etat = 11xxxxxx !
	movlw	7
	movwf	cpt_clig42		; Boucle de 0,2 sec.
	movf	varale1s+1,w		; Dlai d'attente alatoire.
	andlw	31
	addlw	5				; (5  36)*0,2 sec = 1  7 secondes
	movwf	cpt_clig43		; Temps d'attente entre l'arrt du gyrophare clignotant et des phares.
	goto	clig4_fin	
clig4_004
	btfss	status_pha,6	; Etat = 11xxxxxx ?
	goto	clig4_fin
	btfss	status_pha,7
	goto	clig4_fin
; Phares allums en attente d'extinction.
	decfsz	cpt_clig41,f	; On passe ici toutes les 120 s.
	goto	clig4_fin
	decfsz	cpt_clig42,f	; On passe ici toutes les 30 msec = 256*120 s.
	goto	clig4_fin
	movlw	7
	movwf	cpt_clig42		; Pour une boucle de 0,2 sec.
	decfsz	cpt_clig43,f	; On passe ici toutes les 0,2 sec
	goto	clig4_fin		
	bcf		status_pha,6	; Etat = 00xxxxxx !
	bcf		status_pha,7
	bcf		PORTC,4			; Arrt des phares 4.
	movlw	32
	movwf	cpt_clig42		; Pour une boucle de 1 sec.
	movf	varale2s+1,w		; Dlai alatoire d'arrt longue dure.
	andlw	31
	addlw	CLI_PO4			; (0  31) + CLI_PO4(5  220) secondes = 5  251 sec.
	movwf	cpt_clig43
clig4_fin	
	
; ----------------------------------------------------------------------------
; Lampes alatoires
; Le potentiomtre variable permet un ajustement du dlai moyen.
; Dlai = Pot (0-128) + alatoire (0-128)
; Dlai moyen mini = 10 secondes
; Dlai moyen      = 20 secondes
; Dlai moyen maxi = 30 secondes
;	mode_fonc1	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
				; bit(1) =1/0 => Alatoire   4 et 16 = ON/OFF
				; bit(2) =1/0 => Alatoire   5    8 = ON/OFF
				; bit(3) =1/0 => Alatoire  11 et 14 = ON/OFF
alea_debut
	btfss	status_fluo,5	; Zro lampe ?
	goto	alea_fin		; Pas de lampe  animer.
	decfsz	cpt_alea01,f 	; 
	goto	alea_fin
	decfsz	cpt_alea02,f	; On passe ici toutes les 30 ms (256*120 s).
	goto	alea_fin
	movlw	5				; On passe ici toutes les 0,155 secondes, soit 40 sec max pour 255.
	movwf	cpt_alea02
; ------------------------------------
; Lecture CAN = 0 - 255 => Temporisation moyenne 30 secondes pour mi-course.
;                          = 127*0.250 = 31
alea_1_3_15
	btfss	mode_fonc1,0	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
	goto	alea_4_16
	decfsz	cpt_lamp01,f
	goto	alea_1_3_15_a
	movlw	b'00100000'
	xorwf	PORTA,f			; Inverse l'tat de la lampe.
	movf	varale1s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp01		; 7  255 = 1  40 secondes entre 2 changements d'tat.
alea_1_3_15_a
	decfsz	cpt_lamp02,f
	goto	alea_1_3_15_b
	movlw	b'00010000'
	xorwf	PORTA,f			; Inverse l'tat de la lampe.
	movf	varale2s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp02
alea_1_3_15_b
	decfsz	cpt_lamp03,f
	goto	alea_1_3_15_c
	movlw	b'00100000'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale1s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp03
alea_1_3_15_c
	decfsz	cpt_lamp15,f
	goto	alea_4_16
	movlw	b'00100000'
	xorwf	PORTB,f			; Inverse l'tat de la lampe.
	movf	varale2s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp15
; ------------------------------------
alea_4_16
	btfss	mode_fonc1,1	; bit(1) =1/0 => Alatoire 4 et 16 = ON/OFF
	goto	alea_5_8
	decfsz	cpt_lamp04,f
	goto	alea_4_16_a
	movlw	b'00010000'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale1s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp04
alea_4_16_a
	decfsz	cpt_lamp16,f
	goto	alea_5_8
	movlw	b'01000000'
	xorwf	PORTB,f			; Inverse l'tat de la lampe.
	movf	varale2s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp16
; ------------------------------------
alea_5_8
	btfss	mode_fonc1,2	; bit(2) =1/0 => Alatoire 5  8 = ON/OFF
	goto	alea_11_14
	decfsz	cpt_lamp05,f
	goto	alea_5_8_a
	movlw	b'00001000'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale1s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp05
alea_5_8_a
	decfsz	cpt_lamp06,f
	goto	alea_5_8_b
	movlw	b'01000000'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale2s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp06
alea_5_8_b
	decfsz	cpt_lamp07,f
	goto	alea_5_8_c
	movlw	b'10000000'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale1s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp07
alea_5_8_c
	decfsz	cpt_lamp08,f
	goto	alea_11_14
	movlw	b'10000000'
	xorwf	PORTB,f			; Inverse l'tat de la lampe.
	movf	varale2s,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp08
; ------------------------------------
alea_11_14
	btfss	mode_fonc1,3	; bit(3) =1/0 => Alatoire  11 et 14 = ON/OFF
	goto	alea_fin
	decfsz	cpt_lamp11,f
	goto	alea_11_14_a
	movlw	b'00000001'
	xorwf	PORTC,f			; Inverse l'tat de la lampe.
	movf	varale1s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp11
alea_11_14_a
	decfsz	cpt_lamp14,f
	goto	alea_fin
	movlw	b'00010000'
	xorwf	PORTB,f			; Inverse l'tat de la lampe.
	movf	varale2s+1,w
	andlw	0x7F
	addwf	can_lue,w
	movwf	cpt_lamp14
alea_fin
;
; ----------------------------------------------------------------------------
; Lecture CAN
can_debut
	decfsz	cpt_can01,f		; On ne lit le potentiomtre qu'une fois sur 256 = 32 msec.
	goto	can_fin
	btfsc	ADCON0,GO		; Nouveau calcul de cptlen = Lecture valeur du potentiomtre entre 0 et 255.	
	goto	$-1 			; Normalement la dernire conversion est finie depuis longtemps.
	movf	ADRESH,w
	sublw	255 			; Inverse la valeur, car le potentiomtre est cbl  l'envers sur le circuit imprim.
	movwf	can_lue
	bcf		STATUS,C
	rrf		can_lue,f		; 0-127
	movlw	7
	subwf	can_lue,w
	btfsc	STATUS,C
	goto	can_001
	movlw	7
	movwf	can_lue			; Valeur minimum = 7 => 1 seconde mini entre deux changements d'tat alatoire.
can_001	
	bsf 	ADCON0,GO   	; Relance la prochaine conversion can.
can_fin
	goto	prg_fin

; ----------------------------------------------------------------------------
; Squence Flammes  4 lampes.
; On calcule ici pour les 4 leds.
;
; Flamme 1  4 = PWM alatoire
; cpt_flam00  = Numro d'effet en cours = 0  13.
; cpt_flam01  = Compteur pour passer N fois dans un effet avec les mmes valeurs "cpt_flamX1b".
; cpt_flam02  = Compteur pour passer N fois dans un effet. Valeur lue dans la table "table_flam".
; cpt_flam11  = Valeur de puissance moyenne.
; cpt_flam11b = Valeur de puissance brute.
; ----------------------------------------------------------------------------
flam_debut
	bcf		status_fluo,7	
; On passe ici toutes les 17 msec = 31 ms / 2.
; On est synchrone avec le chargement des valeurs dans la procdure d'interruption.
; On passe ici, si cpt_flam01 = 1,2,3,4,5,6,7,8,9....
	movf	cpt_flam11b,w	; On fait toujours une moyenne pour viter les sursauts de lumire.
	addwf	cpt_flam11,f
	rrf		cpt_flam11,f	; Si dbordement dans l'addition, le bit "CARRY" est positionn et utilis.
	movf	cpt_flam21b,w
	addwf	cpt_flam21,f
	rrf		cpt_flam21,f
	movf	cpt_flam31b,w
	addwf	cpt_flam31,f
	rrf		cpt_flam31,f
	movf	cpt_flam41b,w
	addwf	cpt_flam41,f
	rrf		cpt_flam41,f
f000
	decfsz	cpt_flam01,f	; Compteur pour passer N fois dans un effet avec les mmes valeurs "cpt_flamX1b". 
	goto	flam_fin		; 3,4,5,6,7... => dure de l'effet = 45, 60, 75, 90, 105, 120, 135, 150 msec
	decfsz	cpt_flam02,f	; Compteur pour passer N fois dans un effet. Valeur lue dans la table "table_flam".
	goto	effet0			; On est encore dans la srie du mme effet.
	movf	cpt_flam00,w	; Numro d'effet en cours. On vient de finir la srie du mme effet.
	sublw	7
	btfss	STATUS,Z
	goto	f001
	movlw	11				; Si l'effet tait = 7 => On enchaine avec l'effet = 11.
	movwf	cpt_flam00
	goto	f003
f001
	movf	cpt_flam00,w
	sublw	9
	btfss	STATUS,Z
	goto	f002
	movlw	13				; Si l'effet tait = 9 => On enchaine avec l'effet = 13.
	movwf	cpt_flam00
	goto	f003
f002
	movf	varale1s,w		; Changement alatoire d'effet entre 0 et 10.
	andlw	0x07
	movwf	cpt_flam00		; 0-7
	movf	varale2s,w
	andlw	0x03
	addwf	cpt_flam00,f	; 0  10
f003
	movlw	0x06
	movwf	PCLATH
	movf	cpt_flam00,w	; 0  10 = N du prochain effet.
	call	table_flam
	movwf	cpt_flam02		; cpt_flam02 = 1-255 => Nb de boucles  raliser pour le prochain effet.
	incf	cpt_flam02,f	; Compteur pour passer N fois dans un effet. Valeur lue dans la table "table_flam".
	movlw	1
	movwf	cpt_flam01		; 1 => Pour passer dans l'effet directement.
	goto	flam_fin
effet0
	btfsc	cpt_flam00,0	; Numro d'effet en cours = 0,2,4,8,6,8 ou 10
	goto	effet1			; Effets numros impaires
;---------------------------- EFFET 0, 2, 4, 6, 8 ou 10 : FLAMMES NORMALES. 
; 30  135 msec par boucle.
; 60  80 boucles = 1,8  10 secondes d'animation.
	movf 	varale1s,w
	iorlw	0x04			; Puissance mminimum = 8
	movwf	cpt_flam11b		
	movf 	varale1s+1,w		
	iorlw	0x04
	movwf	cpt_flam21b		
	movf 	varale2s,w		
	iorlw	0x04			
	movwf	cpt_flam31b		
	movf 	varale2s+1,w		
	iorlw	0x04
	movwf	cpt_flam41b		
	movf	varale1s+2,w	; Temporisation fois 15 milli-secondes
	andlw	0x07			; 0-7
	addlw	2
	movwf	cpt_flam01		; 2-9 => 30  135 msec , Temporisation en fois 15 * msec.
	goto	flam_fin
effet1
	movf	cpt_flam00,w	; Numro d'effet en cours = 1?
	sublw	1
	btfss	STATUS,Z
	goto	effet3
;---------------------------- EFFET 1 : ROTATION DES FLAMMES. 
	movf 	varale1s,w
	iorlw	0x10			; Puissance mminimum = 16
	movwf	cpt_flam11b		
	movf 	varale1s+1,w		
	iorlw	0x10		
	movwf	cpt_flam21b		
	movf 	varale2s,w		
	iorlw	0x10			
	movwf	cpt_flam31b		
	movf 	varale2s+1,w		
	iorlw	0x10
	movwf	cpt_flam41b		
	movf	cpt_flam11,w	; On permute les flammes.
	movwf	temp1
	movf	cpt_flam21,w
	movwf	cpt_flam11
	movf	cpt_flam31,w
	movwf	cpt_flam21
	movf	cpt_flam41,w
	movwf	cpt_flam31
	movf	temp1,w
	movwf	cpt_flam41
	movf	varale1s+2,w	; Temporisation fois 15 milli-secondes
	andlw	0x07			; 0-7
	addlw	2
	movwf	cpt_flam01		; 2-9 => 30  135 msec , Temporisation en fois 15 * msec.
	goto	flam_fin
effet3
	movf	cpt_flam00,w	; Numro d'effet en cours = 3 ?
	sublw	3
	btfss	STATUS,Z
	goto	effet5
;---------------------------- EFFET 3 : FLAMMES RAPIDES.
	movf	varale1s,w		;
	movwf	cpt_flam11b		; 0-255 => Flamme 1.
	movf	varale1s+1,w	;
	movwf	cpt_flam21b		; 0-255 => Flamme 2.
	movf	varale2s,w		;
	movwf	cpt_flam31b		; 0-255 => Flamme 3.
	movf	varale2s+1,w	;
	movwf	cpt_flam41b		; 0-255 => Flamme 4.
	movf	varale1s+2,w	; Temporisation en milli-secondes.
	andlw	0x03			; 0-3
	addlw	1
	movwf	cpt_flam01		; 1-4 => 15  60 msec, Temporisation en fois 15 * msec.
	goto	flam_fin
effet5
	movf	cpt_flam00,w	; Numro d'effet en cours = 5 ?
	sublw	5
	btfss	STATUS,Z
	goto	effet7
;---------------------------- EFFET 5 : FLAMMES PUISSANTES.
	movf 	varale1s,w
	iorlw	0x20			; Puissance minimum = 32
	movwf	cpt_flam11b		
	movf 	varale1s+1,w		
	iorlw	0x10			; Puissance minimum = 16
	movwf	cpt_flam21b		
	movf 	varale2s,w		
	iorlw	0x20			
	movwf	cpt_flam31b		
	movf 	varale2s+1,w		
	iorlw	0x10
	movwf	cpt_flam41b		
	movf	varale1s+1,w	; Temporisation en milli-secondes.
	andlw	0x03			; 0-3
	addlw	1
	movwf	cpt_flam01		; 1-4 => 15  60 msec, Temporisation en fois 15 * msec.
	goto	flam_fin
effet7
	movf	cpt_flam00,w	; Numro d'effet en cours = 7 ?
	sublw	7
	btfss	STATUS,Z
	goto	effet9
;---------------------------- EFFET 7 : EXTINCTION PUIS RALLUMAGE LENT + Suivit de l'effet n11.
	movf	cpt_flam02,w	; Compteur pour passer N fois dans un effet. Valeur lue dans la table "table_flam".
effet71						; W = 224 -> 0.
	movwf	temp1			; ici w est renseign, on le met dans temp1. Il faut mettre 225 dans la table !
	movf	varale1s,w
	andlw	0x1F			; 0-31
	addwf	temp1,w			; 224 -> 0 +(0-31)
	movwf	cpt_flam11b		; => Flamme 1.
	movf	varale1s+1,w
	andlw	0x1F			; 0-31
	addwf	temp1,w			; 224 -> 0 +(0-31)
	movwf	cpt_flam21b		; => Flamme 2.
	movf	varale2s,w
	andlw	0x1F			; 0-31
	addwf	temp1,w			; 224 -> 0 +(0-31)
	movwf	cpt_flam31b		; => Flamme 3.
	movf	varale2s+1,w
	andlw	0x1F			; 0-31
	addwf	temp1,w			; 224 -> 0 +(0-31)
	movwf	cpt_flam41b		; => Flamme 4.
	movf	varale1s+2,w	; Si effet n 9 ou n 13 on va vite.
	andlw	0x01			; 0-1
	addlw	1
	movwf	cpt_flam01		; 1 - 2 => 15  30 msec, Temporisation en 15 * ms.
	movf	cpt_flam00,w	; Numro d'effet en cours ?
	sublw	7
	btfsc	STATUS,Z
	incf	cpt_flam01,f	; 2 - 3 => 30  45 msec, Temporisation en 15 * ms.
	movf	cpt_flam00,w
	sublw	11
	btfsc	STATUS,Z
	incf	cpt_flam01,f	; 2 - 3 => 30  45 msec, Temporisation en 15 * ms.
	goto	flam_fin
effet9
	movf	cpt_flam00,w	; Numro d'effet en cours = 9 ?
	sublw	9
	btfss	STATUS,Z
	goto	effet11
;---------------------------- EFFET 9 : EXTINCTION PUIS RALLUMAGE RAPIDE + Suit l'effet n13.
	movf	cpt_flam02,w	; cpt_flam02 = Compteur de boucles pour cet effet 128->0.
	goto	effet71			; On reprend la squence n 7 avec tempo plus faible en sortie.
effet11
	movf	cpt_flam00,w	; Numro d'effet en cours = 11 ?
	sublw	11
	btfss	STATUS,Z
	goto	effet13
;---------------------------- EFFET 11 : RALLUMAGE LENT, suite de l'effet n 7.
	movf	cpt_flam02,w	; cpt_flam02 = Compteur de boucles pour cet effet 224->0.
	movwf	temp1			; cpt_flam02 de 224  0
	comf	temp1,w			; Donne => temp1 et w de 0  224	
	addlw	-31				; Ici w est renseign.
	goto	effet71			; On reprend la squence n 7  l'envers.	
effet13
	movf	cpt_flam00,w	; Numro d'effet en cours = 13 ?
	sublw	13
	btfss	STATUS,Z
	goto	flam_fin
;---------------------------- EFFET 13 : RALLUMAGE RAPIDE, suit l'effet n 9.
	movf	cpt_flam02,w	; cpt_flam02 = Compteur de boucles pour cet effet 128->0.
	movwf	temp1			; cpt_flam02 de 128  0
	comf	temp1,w			; Donne => temp1 et w de 0  128	
	addlw	-128			; Ici w est renseign.
	goto	effet71			; On reprend la squence n 7  l'envers avec tempo plus faible en sortie.
flam_fin
; ----------------------------------------------------------------------------
prg_fin
	goto	boucle			; A activer dans la version finale.
	movf	TMR0,w
	subwf	verif_tmr0,w
	btfsc	STATUS,C
	goto	boucle
	movf	TMR0,w			; Garde la valeur maximum !!! Ne doit pas atteindre 255 !!!
	movwf	verif_tmr0		; Pour voir si l'on peut tout traiter entre 2 interruptions ?  200  255 maximum !
	goto	boucle
;
;*******************************************************************
;*******************************************************************
; Entre w = Numro d'effet pour les flammes (0  13).
; Retour w = Nombre de boucles pour les flammes.
; Utilisation de la table :
; 	movlw	0x06
;	movwf	PCLATH
;	movf	cpt_flam02,w
;	call	table_flam
;   1 sec = 66
;   2 sec = 133
;	3 sec = 200
; 	4 sec = 255
;
	ORG 0x600		; Adresse des tableaux en fin de mmoire (Moins de 256 octets par table).
table_flam:
	addwf	PCL,f	; Ajouter w  PCL
	retlw	120		;  0 Normal
	retlw	48		;  1 Rotation
	retlw	120		;  2 Normal
	retlw	60		;  3 Rapide
	retlw	120		;  4 Normal
	retlw	41		;  5 Puissant
	retlw	120		;  6 Normal
	retlw	225		;  7 Disparait + Rapparait lentement (en n 11). 225 obligatoire.
	retlw	80		;  8 Normal
	retlw	129		;  9 Disparait + Rapparait rapidement (en n 13). 129 obligatoire.
	retlw	80		; 10 Normal
	retlw	225		; 11 Rapparait lentement (suite du n 7) 225 obligatoire.
	retlw	80		; 12 Normal
	retlw	129		; 13 Rapparait rapidement en (suite du n 9). 129 obligatoire.
	retlw	0		; 14 Fin de table
	retlw	0		; 15 Fin de table


;*******************************************************************
; Entre w = Compteur de squence pour 3 tubes fluorescents
; Retour w = Valeurs des 3 lampes = XXXXX210 (Bit = 1 = lampe allume)
;          = Bit(7)= 1 = fin de table => Faire raz compteur.
; Tube Fluo 1 = Nombreuses tentatives d'allumages puis reste allum un certain temps.
; Tube Fluo 2 = Nombreuses tentatives d'allumages mais reste allum qu'un petit moment.
; Tube Fluo 3 = Tentatives d'allumages en continu, ne parvient pas  s'allumer.
; Utilisation de la table :
; 	movlw	0x07
;	movwf	PCLATH
;	movf	cpt_fluo,w	
;	call	table_fluo
	ORG 0x700			; Adresse des tableaux en fin de mmoire (moins de 256 octets par table)
table_fluo:
	addwf	PCL,f		; Ajouter w  PCL
	retlw	b'00000111'	; Retour : Lampes: X,X,X,X, lampe n 3, lampe n 2, lampe n  1.
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000100'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000110'
	retlw	b'00000110'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000101'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'	; Il faut que les dernires valeurs = '0' pour que les tubes soient teints pendant la pause longue.
	retlw	b'10000000'	; Dernire valeur valide. Fin de table => Bit(7) = 1.
	
;***************************************************************************************************************
; Dbut du code d'initialisation en page n1, 0x900 pour laisser de la place au programme
; principal en page n0 0x000-0x7FF.
; Attention, ici les 'call' ne doivent faire appel qu' des sous-programmes en page n1.
; Faire avant un "bsf PCLATH,3" pour prparer le saut vers BANK1 (0x800 - 0xFFF).
;
	ORG     0x900		; Le code dbute sur la page n1 !!!
init_suite
	BANK0
	movlw   b'00100000'	; Interruptions = GIE=0 pour le moment, T0IE=1 (Timer overflow interrupt enable bit).
	movwf   INTCON
	movlw	b'10000001'	; Format de conversion analogique 12 bits cadr  droite sur deux deux octets + Vref=+5V + Convertisseur=actif.
	movwf	ADCON0		; ADFM = 1, ADON = 1.
	BANK1
	movlw   b'01110000'	; 'x111'=Vitesse 8 Mhz + 'x'=OSTS interne + 'x'=HFINTOSC is stable + 'x'=LFINTOSC is stable + '0'=SCS interne.
	movwf	OSCCON		; 'x' = Bit en lecture seulement.
	nop
	nop
	movlw   b'00100000'	; Dure de conversion analogique = 2 s = F/16 pour 8 Mhz.
	movwf   ADCON1
	BANK0
	clrf	PORTA       ; Au dpart, teint toutes les sorties.
	clrf	PORTB
	clrf	PORTC
; T0CS   =   0 = Source interne pour TMR0
; PSA    =   0 = Prdiviseur pour le Timer0
; PS     = 001 = Prdiviseur pour Timer0 = 1/4 => (4*4)/8 Mhz = 2 sec
	BANK1
	movlw   b'11010001'	; PortB, pas de rsistances de rappel, Timer0 Clock Source Select bit = Focs/4,
	movwf	OPTION_REG	; Prescaler is assigned to the Timer0 module.
	movlw   b'00110111'	; '1'=Weak pull-up actif.
	movwf	WPUA
	clrf	PIE1	 	; '0'=Pas d'interruption.
	clrf	PIE2
	clrf	IOCA
	movlw   b'00001001'	; PortA en sortie (4 bits) sauf RA0=Entre CAN0 et RA3=Entre numrique.
	movwf   TRISA
	clrf	TRISB		; PortB en sortie (4 bits).
	clrf	TRISC		; PortC en sortie (8 bits).
	BANK2
	movlw   b'00000001'	; Utilisation de RA0 en entre analogique CAN0.
	movwf	ANSEL
	clrf	ANSELH		; Utilisation de RA0 en entre analogique CAN0.
	clrf	CM1CON0		; Les comparateurs ne sont pas utiliss.
	clrf	CM2CON0	
	BANK0
;*******************************************************************
; Initialisation des variables du programme.
	movlw	0x20 		; Raz de toutes les variables.
	movwf	FSR			; On charge l'adresse de dbut (0x20) de la zone des variables.
clr_var
	clrf	INDF		; Effacement du contenu du registre point par FSR
	incf	FSR,f		; On incrmente l'adresse.
	movlw	0x80		; Adresse de fin + 1 (0x80)
	subwf	FSR,w
	btfss	STATUS,Z	; On teste si (FSR) = 0x80.
	goto	clr_var		; Non : on recommence.
	movlw	1			; 1 => Pour passer tout de suite dans la boucle.
	movwf	cpt_flam02
	movwf	cpt_flam03
	clrf	status_pha
	movlw	1
	movwf	cpt_feup13	; Mise en route des phares rapidement, mais lgrement dcals.
	movlw	3			; Delai en secondes.
	movwf	cpt_feup23
	movlw	5
	movwf	cpt_clig33
	movlw	7
	movwf	cpt_clig43
	movlw	32
	movwf	cpt_feup12	; Boucle de 1 sec.
	movwf	cpt_feup22
	movwf	cpt_clig32
	movwf	cpt_clig42
	movlw	1
	clrf	cpt_feup14
	clrf	cpt_feup15
	clrf	cpt_feup24
	clrf	cpt_feup25

	movlw	2*PEN_NF1
	movwf	cpt_feup16	; Si = 6 => Squence de 3 flashs.
	movlw	2*PEN_NF2
	movwf	cpt_feup26

	movlw	1
	movwf	cpt_arc12	; Fin des boucles de pause longue dure.
	movwf	cpt_arc13
	movwf	cpt_arc22
	movwf	cpt_arc23
	movwf	cpt_arc32
	movwf	cpt_arc33

	movwf	cpt_alea02
	movwf	cpt_alea03	; Fin des boucles de pause longue dure.

	movlw	0x30		; Pour initialiser les gnrateurs de nombre alatoire.
	movwf	varale1s
	movlw	0x45
	movwf	varale1s+1
	movlw	0x54
	movwf	varale2s
	movlw	0x03
	movwf	varale2s+1
	bsf		ADCON0,GO	; Relance la prochaine conversion can.
	nop
	nop
	btfsc	ADCON0,GO    
	goto	$-1
	BANK1
	movf	ADRESL,w	; 8 bits de poids faible sur les 10 bits.
	BANK0
	movwf	can_lue		; 0  255.
	movlw   b'00000001'	; Format de conversion analogique 8 bits les plus forts sur un octet + Vref=+5V + Convertisseur=actif.
	movwf	ADCON0		; ADFM = 0, ADON = 1. La conversion CAN est lanc plus loin.
	movf	can_lue,w	; 0  255.
	andlw	b'00000001'	; Pour modifier les valeurs initiales des gnrateurs de nombre alatoire.
	addwf	varale1s,f  ; pour avoir des diffrences de comportement entre carte identiques.
	movlw	can_lue		; 0  255.
	andlw	b'00000010'
	addwf	varale1s+1,f
	movlw	can_lue		; 0  255.
	andlw	b'00000100'
	addwf	varale2s,f
	movlw	can_lue		; 0  255.
	andlw	b'00001000'
	addwf	varale2s+1,f
	movlw	b'00000001'	; Format de conversion analogique 8 bits cadr  droite sur deux deux octets + Vref=+5V + Convertisseur=actif.
	movwf	ADCON0		; ADFM = 0, ADON = 1.
;
;***********************************************************************************************
; Lecture des cavaliers en entre de R3 et sorties RA5, RA4, RC5 et RC4.
; utilise ici, tvemporairement la variable "cpt_vara01".
	clrf	cpt_vara01
	bsf 	PORTC,4		; Lecture des cavaliers pour choisir le mode de fonctionnement.
	clrf	temp1		; Les autres sorties sont dj  0.
	decfsz	temp1,f
	goto	$-1         ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf 	cpt_vara01,0      ; 0-0-0-1.
	bcf 	PORTC,4
	bsf 	PORTC,5
	decfsz	temp1,f
	goto	$-1         ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf 	cpt_vara01,1 		; 0-0-1-0.
	bcf 	PORTC,5
	bsf 	PORTA,4
	decfsz	temp1,f
	goto	$-1         ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf 	cpt_vara01,2 		; 0-1-0-0.
	bcf 	PORTA,4
	bsf 	PORTA,5
	decfsz	temp1,f
	goto	$-1         ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf 	cpt_vara01,3		; 1-0-0-0.
	bsf 	ADCON0,GO	; Lance la conversion can pour la prochaine lecture.
	bcf 	PORTA,5		; Au dpart, teint toutes les sorties.
; cpt_vara01	; Mode de fonctionnement du programme = Lecture des cavaliers 1  4.
;	mode_fonc1	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
				; bit(1) =1/0 => Alatoire   4 et 16 = ON/OFF
				; bit(2) =1/0 => Alatoire   5    8 = ON/OFF
				; bit(3) =1/0 => Alatoire  11 et 14 = ON/OFF
; mode_fonc2	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
				; bit(1) =1/0 => Flammes = ON/OFF
				; bit(2) =1/0 => Feux pntration et phares 1 & 2 = ON/OFF
				; bit(3) =1/0 => Feux clignotants et phares 3 = ON/OFF
				; bit(4) =1/0 => Feux clignotants et phares 4 = ON/OFF
				; bit(5) =1/0 => Arc 3  = ON/OFF
				; bit(6) =1/0 => Fluo 3 = ON/OFF
				; bit(7) =1/0 => Phares 1 & 2 = ON/OFF
	clrf	mode_fonc1		; Positionnement des indicateurs de fonctionnement.
	clrf	mode_fonc2
	btfss	cpt_vara01,0	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
	bsf 	mode_fonc1,0	
	btfss	cpt_vara01,1
	bsf 	mode_fonc1,0	

	btfss	cpt_vara01,3 	; bit(1) =1/0 => Alatoire 4 et 16 = ON/OFF
	bsf 	mode_fonc1,1
	bsf 	mode_fonc1,2	
	btfsc	cpt_vara01,0	; bit(2) =1/0 => Alatoire 5  8 = ON/OFF
	bcf 	mode_fonc1,2	
	btfsc	cpt_vara01,1		
	bcf 	mode_fonc1,2	

	btfss	cpt_vara01,2	; bit(3) =1/0 => Alatoire 11 et 14 = ON/OFF
	bsf 	mode_fonc1,3

	bsf 	mode_fonc2,0	; bit(0) =1/0 => Arc 1&2 & Fluo 1&2 = ON/OFF
	btfsc	cpt_vara01,0 
	bsf 	mode_fonc2,1
	btfsc	cpt_vara01,1
	bcf 	mode_fonc2,1	; bit(1) =1/0 => Flammes = ON/OFF
	btfsc	cpt_vara01,1
	bsf 	mode_fonc2,2	; bit(2) =1/0 => Feux pntration 1 & 2 = ON/OFF

	bsf 	mode_fonc2,3
	btfss	cpt_vara01,0 
	bcf 	mode_fonc2,3
	btfss	cpt_vara01,1
	bcf 	mode_fonc2,3	; bit(3) =1/0 => Feux clignotants 3 & phares 3 = ON/OFF

	bsf 	mode_fonc2,7
	btfss	cpt_vara01,0 
	bcf 	mode_fonc2,7
	btfss	cpt_vara01,1
	bcf 	mode_fonc2,7	; bit(7) =1/0 => Phares 1 & 2 = ON/OFF

	btfsc	cpt_vara01,3
	bsf 	mode_fonc2,4	; bit(4) =1/0 => Feux clignotants 4 & phares 4 = ON/OFF
	btfsc	cpt_vara01,2
	bsf 	mode_fonc2,5	; bit(5) =1/0 => Arc 3 = ON/OFF
	btfsc	cpt_vara01,2
	bsf 	mode_fonc2,6	; bit(6) =1/0 => Fluo 3 = ON/OFF
	
;	mode_fonc1	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
				; bit(1) =1/0 => Alatoire   4 et 16 = ON/OFF
				; bit(2) =1/0 => Alatoire   5    8 = ON/OFF
				; bit(3) =1/0 => Alatoire  11 et 14 = ON/OFF
	movlw	1
	btfsc	mode_fonc1,0
	movwf	cpt_lamp01	;	Pour un dcalage obligatoire entre lampes au dpart.
	movlw	3
	btfsc	mode_fonc1,0
	movwf	cpt_lamp02	;	Attention de ne pas rentrer en contradiction avec 
	movlw	5
	btfsc	mode_fonc1,0
	movwf	cpt_lamp03	;   les autres variables qui partagent les mmes adresses !
	movlw	7
	btfsc	mode_fonc1,1
	movwf	cpt_lamp04	
	movlw	9
	btfsc	mode_fonc1,2
	movwf	cpt_lamp05	
	movlw	11
	btfsc	mode_fonc1,2
	movwf	cpt_lamp06	
	movlw	13
	btfsc	mode_fonc1,2
	movwf	cpt_lamp07	
	movlw	15
	btfsc	mode_fonc1,2
	movwf	cpt_lamp08	
	movlw	17
	btfsc	mode_fonc1,3
	movwf	cpt_lamp11	
	movlw	19
	btfsc	mode_fonc1,3
	movwf	cpt_lamp14	
	movlw	21
	btfsc	mode_fonc1,0
	movwf	cpt_lamp15	
	movlw	23
	btfsc	mode_fonc1,1
	movwf	cpt_lamp16	
;-----------------------------	
; Dcalage pour ne pas passer dans toutes les sous-programmes en mme temps.
	movlw	1
	movwf	cpt_feup11
	movlw	10
	movwf	cpt_feup21
	movlw	20
	movwf	cpt_arc11
	movlw	30
	movwf	cpt_arc21
	movlw	40
	movwf	cpt_arc31
	movlw	50
	movwf	cpt_fluo11
	movlw	60
	movwf	cpt_fluo21
	movlw	80
	movwf	cpt_fluo31
	movlw	90
	movwf	cpt_clig31
	movlw	100
	movwf	cpt_clig41
	movlw	110
	movwf	cpt_flam01
	movlw	120
	movwf	cpt_alea01
	movlw	130
	movwf	cpt_can01
	movlw	140
	movwf	cpt_vara01
;-----------------------------	
; Calcul du nombre de lampes alatoires allumes.
;	mode_fonc1	; bit(0) =1/0 => Alatoire 1  3 et 15 = ON/OFF
				; bit(1) =1/0 => Alatoire   4 et 16 = ON/OFF
				; bit(2) =1/0 => Alatoire   5    8 = ON/OFF
				; bit(3) =1/0 => Alatoire  11 et 14 = ON/OFF
	btfsc	mode_fonc1,0
	bsf		status_fluo,5	; Au moins une lampe est allume.
	btfsc	mode_fonc1,1
	bsf		status_fluo,5	; Au moins une lampe est allume.
	btfsc	mode_fonc1,2
	bsf		status_fluo,5	; Au moins une lampe est allume.
	btfsc	mode_fonc1,3
	bsf		status_fluo,5	; Au moins une lampe est allume.
	
	return
	end